package net.wanji.opt.controller.strategy;

import com.github.pagehelper.PageInfo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiResponse;
import io.swagger.annotations.ApiResponses;
import net.wanji.common.framework.rest.JsonViewObject;
import net.wanji.opt.dto.IntegerIdsDTO;
import net.wanji.opt.dto.strategy.AddOrUpdateIdeaDTO;
import net.wanji.opt.dto.strategy.QueryIdeaDTO;
import net.wanji.opt.po.strategy.IdeaPO;
import net.wanji.opt.service.strategy.IdeaService;
import org.springframework.dao.DuplicateKeyException;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;
import javax.ws.rs.core.MediaType;

/**
 * 策略管理-方法库
 *
 * @author Kent HAN
 * @date 2023/2/27 9:25
 */
@Api(value = "IdeaController", description = "策略管理-方法库")
@RequestMapping("/idea")
@RestController
public class IdeaController {
    private final IdeaService ideaService;

    public IdeaController(IdeaService ideaService) {
        this.ideaService = ideaService;
    }

    @ApiOperation(value = "新增/修改方法", notes = "新增/修改方法", response = JsonViewObject.class,
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @PostMapping(value = "/addOrUpdateIdea",
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @ApiResponses({
            @ApiResponse(code = 200, message = "OK", response = JsonViewObject.class),
    })
    public JsonViewObject addOrUpdateIdea(@RequestBody @Valid AddOrUpdateIdeaDTO addOrUpdateIdeaDTO) {
        try {
            ideaService.addOrUpdateIdea(addOrUpdateIdeaDTO);
        } catch (DuplicateKeyException e) {
            throw new DuplicateKeyException("方法编号或方法名称不可重复");
        }
        return JsonViewObject.newInstance().success();
    }

    @ApiOperation(value = "删除方法", notes = "删除方法", response = JsonViewObject.class,
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @PostMapping(value = "/deleteIdea",
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @ApiResponses({
            @ApiResponse(code = 200, message = "OK", response = JsonViewObject.class),
    })
    public JsonViewObject deleteIdea(@RequestBody IntegerIdsDTO integerIdsDTO) {
        ideaService.deleteIdea(integerIdsDTO);
        JsonViewObject jsonViewObject = JsonViewObject.newInstance();

        return jsonViewObject.success();
    }

    @ApiOperation(value = "查询方法", notes = "查询方法", response = JsonViewObject.class,
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @PostMapping(value = "/queryIdea",
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @ApiResponses({
            @ApiResponse(code = 200, message = "OK", response = IdeaPO.class),
    })
    public JsonViewObject queryIdea(@RequestBody QueryIdeaDTO queryIdeaDTO) {
        PageInfo<IdeaPO> ideaPOList = ideaService.queryIdea(queryIdeaDTO);
        JsonViewObject jsonViewObject = JsonViewObject.newInstance();

        return jsonViewObject.success(ideaPOList);
    }
}
