package net.wanji.opt.controller.strategy;

import com.github.pagehelper.PageInfo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiResponse;
import io.swagger.annotations.ApiResponses;
import net.wanji.common.framework.rest.JsonViewObject;
import net.wanji.opt.dto.IntegerIdsDTO;
import net.wanji.opt.dto.strategy.AddOrUpdateStrategyDTO;
import net.wanji.opt.dto.strategy.QueryStrategyDTO;
import net.wanji.opt.po.strategy.StrategyPO;
import net.wanji.opt.service.strategy.StrategyService;
import org.springframework.dao.DuplicateKeyException;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;
import javax.ws.rs.core.MediaType;

/**
 * 策略管理-策略库
 *
 * @author Kent HAN
 * @date 2023/2/27 9:25
 */
@Api(value = "StrategyController", description = "策略管理-策略库")
@RequestMapping("/strategy")
@RestController
public class StrategyController {
    private final StrategyService strategyService;

    public StrategyController(StrategyService strategyService) {
        this.strategyService = strategyService;
    }

    @ApiOperation(value = "新增/修改策略", notes = "新增/修改策略", response = JsonViewObject.class,
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @PostMapping(value = "/addOrUpdateStrategy",
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @ApiResponses({
            @ApiResponse(code = 200, message = "OK", response = JsonViewObject.class),
    })
    public JsonViewObject addOrUpdateStrategy(@RequestBody @Valid AddOrUpdateStrategyDTO addOrUpdateStrategyDTO) {
        try {
            strategyService.addOrUpdateStrategy(addOrUpdateStrategyDTO);
        } catch (DuplicateKeyException e) {
            throw new DuplicateKeyException("策略编号或策略名称不可重复");
        }
        return JsonViewObject.newInstance().success();
    }

    @ApiOperation(value = "删除策略", notes = "删除策略", response = JsonViewObject.class,
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @PostMapping(value = "/deleteStrategy",
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @ApiResponses({
            @ApiResponse(code = 200, message = "OK", response = JsonViewObject.class),
    })
    public JsonViewObject deleteStrategy(@RequestBody IntegerIdsDTO integerIdsDTO) {
        strategyService.deleteStrategy(integerIdsDTO);
        JsonViewObject jsonViewObject = JsonViewObject.newInstance();

        return jsonViewObject.success();
    }

    @ApiOperation(value = "查询策略", notes = "查询策略", response = JsonViewObject.class,
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @PostMapping(value = "/queryStrategy",
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @ApiResponses({
            @ApiResponse(code = 200, message = "OK", response = StrategyPO.class),
    })
    public JsonViewObject queryStrategy(@RequestBody QueryStrategyDTO queryStrategyDTO) {
        PageInfo<StrategyPO> strategyPOList = strategyService.queryStrategy(queryStrategyDTO);
        JsonViewObject jsonViewObject = JsonViewObject.newInstance();

        return jsonViewObject.success(strategyPOList);
    }
}
