package net.wanji.opt.service.impl;

import com.github.pagehelper.PageInfo;
import lombok.extern.slf4j.Slf4j;
import net.wanji.common.dto.CrossIdDTO;
import net.wanji.common.utils.tool.CrossUtil;
import net.wanji.databus.dao.entity.RidInfoEntity;
import net.wanji.databus.dao.mapper.RidInfoMapper;
import net.wanji.opt.dao.mapper.CrossDirInfoMapper;
import net.wanji.opt.dao.mapper.CrossInfoMapper;
import net.wanji.opt.dao.mapper.CrossPhaseMapper;
import net.wanji.opt.dao.mapper.CrossSchemeMapper;
import net.wanji.opt.dao.mapper.CrossSchemeOptLogMapper;
import net.wanji.opt.dao.mapper.strategy.SceneMapper;
import net.wanji.opt.dao.mapper.trend.CrossDataRealtimeMapper;
import net.wanji.opt.dto.CrossIdAndSchemeIdDTO;
import net.wanji.opt.dto.strategy.AddOrUpdateSceneDTO;
import net.wanji.opt.dto.strategy.QuerySceneDTO;
import net.wanji.opt.po.base.CrossInfoPO;
import net.wanji.opt.po.base.CrossPhasePO;
import net.wanji.opt.po.base.CrossSchemeOptLogPO;
import net.wanji.opt.po.base.CrossSchemePO;
import net.wanji.opt.po.strategy.ScenePO;
import net.wanji.opt.po.trend.CrossDataRealtimePO;
import net.wanji.opt.service.DiagnoService;
import net.wanji.opt.service.strategy.SceneService;
import net.wanji.opt.vo.CrossIdAndLocationVO;
import net.wanji.opt.vo.SchemeOptVO;
import org.jetbrains.annotations.NotNull;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;
import java.util.Objects;
import java.util.Optional;

/**
 * @author Kent HAN
 * @date 2023/3/2 10:09
 */
@Service
@Slf4j
public class DiagnoServiceImpl implements DiagnoService {

    private final CrossInfoMapper crossInfoMapper;
    private final CrossDataRealtimeMapper crossDataRealtimeMapper;
    private final SceneService sceneService;
    private final SceneMapper sceneMapper;
    private final CrossDirInfoMapper crossDirInfoMapper;
    private final RidInfoMapper ridInfoMapper;
    private final CrossSchemeOptLogMapper crossSchemeOptLogMapper;
    private final CrossSchemeMapper crossSchemeMapper;
    private final CrossPhaseMapper crossPhaseMapper;

    public DiagnoServiceImpl(CrossInfoMapper crossInfoMapper, CrossDataRealtimeMapper crossDataRealtimeMapper,
                             SceneService sceneService, SceneMapper sceneMapper, CrossDirInfoMapper crossDirInfoMapper,
                             RidInfoMapper ridInfoMapper, CrossSchemeOptLogMapper crossSchemeOptLogMapper,
                             CrossSchemeMapper crossSchemeMapper, CrossPhaseMapper crossPhaseMapper) {
        this.crossInfoMapper = crossInfoMapper;
        this.crossDataRealtimeMapper = crossDataRealtimeMapper;
        this.sceneService = sceneService;
        this.sceneMapper = sceneMapper;
        this.crossDirInfoMapper = crossDirInfoMapper;
        this.ridInfoMapper = ridInfoMapper;
        this.crossSchemeOptLogMapper = crossSchemeOptLogMapper;
        this.crossSchemeMapper = crossSchemeMapper;
        this.crossPhaseMapper = crossPhaseMapper;
    }

    @Override
    public AddOrUpdateSceneDTO queryCrossScene(CrossIdDTO crossIdDTO) {
        // 查询路口场景代码
        String crossId = crossIdDTO.getCrossId();
        CrossDataRealtimePO crossDataRealtimePO = crossDataRealtimeMapper.selectByCrossId(crossId);
        Integer status = crossDataRealtimePO.getStatus();
        // 获取场景名称
        ScenePO scenePO = sceneMapper.selectBySceneNum(status);
        String sceneName = scenePO.getSceneName();
        // 构造请求体
        QuerySceneDTO querySceneDTO = new QuerySceneDTO();
        querySceneDTO.setSceneName(sceneName);
        querySceneDTO.setSceneTarget(1);
        querySceneDTO.setPageNum(1);
        querySceneDTO.setPageSize(999);
        PageInfo<AddOrUpdateSceneDTO> pageInfo = sceneService.queryScene(querySceneDTO);
        List<AddOrUpdateSceneDTO> list = pageInfo.getList();
        AddOrUpdateSceneDTO addOrUpdateSceneDTO = list.get(0);

        return addOrUpdateSceneDTO;
    }

    @Override
    public List<CrossIdAndLocationVO> queryNeighborCross(CrossIdDTO crossIdDTO) {
        List<CrossIdAndLocationVO> crossIdAndLocationVOList = new ArrayList<>();
        String crossId = crossIdDTO.getCrossId();
        List<Integer> inDirs = crossDirInfoMapper.selectInDirsByCrossId(crossId);
        for (Integer inDir : inDirs) {
            CrossIdAndLocationVO crossIdAndLocationVO = new CrossIdAndLocationVO();
            RidInfoEntity ridInfo = ridInfoMapper.selectByEndInDir(crossId, inDir);
            if (ridInfo != null) {
                // 关联路口坐标
                String startCrossId = ridInfo.getStartCrossId();
                crossIdAndLocationVO.setCrossId(startCrossId);
                CrossInfoPO crossInfoPO = crossInfoMapper.selectById(startCrossId);
                if (crossInfoPO != null) {
                    String startLocationStr = crossInfoPO.getLocation();
                    double[] startLonLat = CrossUtil.getLonLat(startLocationStr);
                    List<Double> startLocation = new ArrayList<>();
                    startLocation.add(startLonLat[0]);
                    startLocation.add(startLonLat[1]);
                    crossIdAndLocationVO.setLocation(startLocation);
                }
            }
            crossIdAndLocationVOList.add(crossIdAndLocationVO);
        }
        return crossIdAndLocationVOList;
    }

    @Override
    public SchemeOptVO querySchemeOpt(CrossIdDTO crossIdDTO) {
        String crossId = crossIdDTO.getCrossId();
        SchemeOptVO schemeOptVO = new SchemeOptVO();
        schemeOptVO.setCrossId(crossId);
        List<CrossSchemeOptLogPO> crossSchemeOptLogPOList = crossSchemeOptLogMapper.selectByCrossId(crossId);
        // 选择数据批次时间最大的记录
        Optional<CrossSchemeOptLogPO> max = crossSchemeOptLogPOList.stream()
                .max(Comparator.comparing(CrossSchemeOptLogPO::getDataBatchTime));
        List<CrossSchemeOptLogPO> maxList = getMaxList(crossSchemeOptLogPOList, max);
        CrossSchemeOptLogPO crossSchemeOptLogPO = maxList.get(0);
        String optType = crossSchemeOptLogPO.getOptType();
        schemeOptVO.setOptType(Integer.parseInt(optType));
        String schemeNo = crossSchemeOptLogPO.getSchemeNo();
        CrossSchemePO crossSchemePO = crossSchemeMapper.selectByCrossIdAndSchemeNo(crossId, schemeNo);
        schemeOptVO.setCycle(crossSchemePO.getCycle());
        schemeOptVO.setOffset(crossSchemeOptLogPO.getOffset());
        // 构造调整时间列表
        List<SchemeOptVO.PhaseOptTime> phaseOptTimeList = buildPhaseOptTimeList(maxList);
        schemeOptVO.setPhaseOptTimeList(phaseOptTimeList);
        return schemeOptVO;
    }

    @Override
    public List<SchemeOptVO.PhaseOptTime> queryOptTime(CrossIdAndSchemeIdDTO crossIdAndSchemeIdDTO) {
        String crossId = crossIdAndSchemeIdDTO.getCrossId();
        Integer schemeId = crossIdAndSchemeIdDTO.getSchemeId();
        List<SchemeOptVO.PhaseOptTime> phaseOptTimeList = new ArrayList<>();
        List<CrossSchemeOptLogPO> crossSchemeOptLogPOList = crossSchemeOptLogMapper.selectByCrossId(crossId);
        // 选择数据批次时间最大的记录
        Optional<CrossSchemeOptLogPO> max = crossSchemeOptLogPOList.stream()
                .max(Comparator.comparing(CrossSchemeOptLogPO::getDataBatchTime));
        List<CrossSchemeOptLogPO> maxList = getMaxList(crossSchemeOptLogPOList, max);
        for (CrossSchemeOptLogPO crossSchemeOptLogPO : maxList) {
            SchemeOptVO.PhaseOptTime phaseOptTime = new SchemeOptVO.PhaseOptTime();
            String phaseNo = crossSchemeOptLogPO.getPhaseNo();
            phaseOptTime.setPhaseNo(phaseNo);
            Integer oriGreenTime = crossSchemeOptLogPO.getOriGreenTime();
            phaseOptTime.setOriGreenTime(oriGreenTime);
            Integer optGreenTime = getNewGreenTime(crossId, schemeId, phaseNo);
            phaseOptTime.setOptGreenTime(optGreenTime);
            phaseOptTime.setOptTime(optGreenTime - oriGreenTime);
            phaseOptTimeList.add(phaseOptTime);
        }
        return phaseOptTimeList;
    }

    private Integer getNewGreenTime(String crossId, Integer schemeId, String phaseNo) {
        List<CrossPhasePO> crossPhasePOList = crossPhaseMapper.listCrossPhase(crossId, String.valueOf(schemeId));
        for (CrossPhasePO crossPhasePO : crossPhasePOList) {
            String newPhaseNo = crossPhasePO.getPhaseNo();
            if (Objects.equals(phaseNo, newPhaseNo)) {
                return crossPhasePO.getGreenTime();
            }
        }
        return 0;
    }

    @NotNull
    private static List<CrossSchemeOptLogPO> getMaxList(List<CrossSchemeOptLogPO> crossSchemeOptLogPOList,
                                                        Optional<CrossSchemeOptLogPO> max) {
        int maxBatchTime = max.get().getDataBatchTime();
        List<CrossSchemeOptLogPO> maxList = new ArrayList<>();
        for (CrossSchemeOptLogPO crossSchemeOptLogPO : crossSchemeOptLogPOList) {
            int dataBatchTime = crossSchemeOptLogPO.getDataBatchTime();
            if (dataBatchTime == maxBatchTime) {
                maxList.add(crossSchemeOptLogPO);
            }
        }
        return maxList;
    }

    private List<SchemeOptVO.PhaseOptTime> buildPhaseOptTimeList(List<CrossSchemeOptLogPO> crossSchemeOptLogPOList) {
        List<SchemeOptVO.PhaseOptTime> phaseOptTimeList = new ArrayList<>();
        for (CrossSchemeOptLogPO crossSchemeOptLogPO : crossSchemeOptLogPOList) {
            SchemeOptVO.PhaseOptTime phaseOptTime = new SchemeOptVO.PhaseOptTime();
            phaseOptTime.setPhaseNo(crossSchemeOptLogPO.getPhaseNo());
            phaseOptTime.setOriGreenTime(crossSchemeOptLogPO.getOriGreenTime());
            phaseOptTime.setOptGreenTime(crossSchemeOptLogPO.getOptGreenTime());
            phaseOptTime.setOptTime(crossSchemeOptLogPO.getOptTime());
            phaseOptTimeList.add(phaseOptTime);
        }
        return phaseOptTimeList;
    }
}
