package net.wanji.utc.controller;

import com.github.pagehelper.PageInfo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiResponse;
import io.swagger.annotations.ApiResponses;
import net.wanji.common.annotation.aspect.AspectLog;
import net.wanji.common.enums.BaseEnum;
import net.wanji.common.framework.rest.JsonViewObject;
import net.wanji.utc.dto.systemadmin.DeleteByIntegerIdListDTO;
import net.wanji.utc.dto.systemadmin.ManufacturerInsertOrUpdateDTO;
import net.wanji.utc.dto.systemadmin.ManufacturerListDTO;
import net.wanji.utc.po.ManufacturerInfoPO;
import net.wanji.utc.service.systemadmin.ManufacturerService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.ws.rs.core.MediaType;
import java.util.Objects;

/**
 * 系统管理-厂商管理
 *
 * @author Kent HAN
 * @date 2022/11/24 8:32
 */

@Api(value = "系统管理-厂商管理", description = "系统管理-厂商管理")
@RequestMapping("/manufacturer")
@RestController
public class ManufacturerController {
    @Autowired
    ManufacturerService manufacturerService;

    @AspectLog(description = "厂商列表", operationType = BaseEnum.OperationTypeEnum.QUERY)
    @ApiOperation(value = "厂商列表", notes = "厂商列表", response = ManufacturerInfoPO.class,
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @PostMapping(value = "/list",
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @ApiResponses({
            @ApiResponse(code = 200, message = "OK", response = ManufacturerInfoPO.class),
    })
    public JsonViewObject list(@RequestBody ManufacturerListDTO manufacturerListDTO) {
        Integer pageNum = manufacturerListDTO.getPageNum();
        Integer pageSize = manufacturerListDTO.getPageSize();
        // 不分页
        if (Objects.isNull(pageNum) || pageNum <= 0) {
            manufacturerListDTO.setPageNum(1);
        }
        if (Objects.isNull(pageSize) || pageSize <= 0) {
            manufacturerListDTO.setPageSize(9999);
        }
        PageInfo<ManufacturerInfoPO> manufacturerInfoPOPageInfo = manufacturerService.list(manufacturerListDTO);
        JsonViewObject jsonViewObject = JsonViewObject.newInstance();
        return jsonViewObject.success(manufacturerInfoPOPageInfo);
    }

    @AspectLog(description = "厂商添加或修改", operationType = BaseEnum.OperationTypeEnum.UPDATE)
    @ApiOperation(value = "厂商添加或修改", notes = "厂商添加或修改", consumes = MediaType.APPLICATION_JSON)
    @PostMapping(value = "/insertOrUpdate", consumes = MediaType.APPLICATION_JSON)
    public JsonViewObject insertOrUpdate(@RequestBody @Validated ManufacturerInsertOrUpdateDTO inVO) {
        manufacturerService.insertOrUpdate(inVO);
        JsonViewObject jsonViewObject = JsonViewObject.newInstance();

        return jsonViewObject.success();
    }

    @AspectLog(description = "厂商删除", operationType = BaseEnum.OperationTypeEnum.DELETE)
    @ApiOperation(value = "厂商删除", notes = "厂商删除", consumes = MediaType.APPLICATION_JSON)
    @PostMapping(value = "/delete", consumes = MediaType.APPLICATION_JSON)
    public JsonViewObject delete(@RequestBody DeleteByIntegerIdListDTO inVO) {
        manufacturerService.delete(inVO);
        JsonViewObject jsonViewObject = JsonViewObject.newInstance();

        return jsonViewObject.success();
    }
}
